/*
  lupin.c  -- show subtitles in 'Lupin the 3rd.'

   Original(X68K) version written by UNKNOWN (please tell me...)
   PC98 version written by KONO Takeshi.
   PC/AT version written by AOI-Hoshi
   Windows version written by shimarin (LupinWIN)
      and Hiroyuki Takeda (Lupin Subtitler)
   HP-100/LX version written by AOI-Hoshi

   SDL version written by Zinnia (Tsuyoshi Iguchi)

$Id$
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iconv.h>
#include <time.h>

#include "SDL.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"

#include "lupin_title.h"

#ifdef ENABLE_AUDIO
static int audio_open = 0;
#endif

static char rcsid[] = "$Id$";

static SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
static SDL_Color black = { 0x00, 0x00, 0x00, 0 };
static SDL_Surface* text[10];  /* Rendered font spool */

typedef enum { ALIGN_CENTER, ALIGN_RIGHT, ALIGN_LEFT } align;

/* atexit */
void CleanUp(void)
{
#ifdef ENABLE_AUDIO
  if(audio_open) {
    Mix_CloseAudio();
    audio_open = 0;
  }
#endif
  SDL_Quit();
}

/* Render 1 line */
int  prepare_line(iconv_t cd, TTF_Font* font, char* str)
{
  char* p;
  char* p1, * p2;
  char intmp[3], outtmp[10];
  int slen, dlen;
  int nconv;
  int i;
  for(p = str, i = 0; i < 10 && *p != '\0';){
    if(*p != ' '){
      memcpy(intmp, p, 2);
      intmp[2] = '\0';

      slen = 3; dlen = 10;
      p1 = intmp; p2 = outtmp;
      nconv = iconv(cd, (const char**)&p1, &slen, (char**)&p2, &dlen);
      if(nconv == -1){
	fprintf(stderr, "lupin: iconv: error.\n");
	exit(1);
      }

      text[i] = TTF_RenderUTF8_Solid(font,outtmp,white);
      if ( text == NULL ) {
	fprintf(stderr, "lupin: TTF_RenderUTF8_Solid: %s\n", SDL_GetError());
	exit(1);
      }
      p += 2;
      i++;
    } else {
      p++;
    }
  }
  text[i] = NULL;
  return i;
}

/* Show 1 letter */
void showletter(SDL_Surface* screen, int count)
{
  SDL_Rect dstrect;
  SDL_Rect rgn;
  if(text[count] == NULL) return;

  dstrect.x = (screen->w - text[count]->w)/2;
  dstrect.y = (screen->h - text[count]->h)/2;
  dstrect.w = text[count]->w;
  dstrect.h = text[count]->h;
  rgn = dstrect;

  /* Blit the text surface */
  if ( SDL_BlitSurface(text[count], NULL, screen, &dstrect) < 0 ) {
    fprintf(stderr, "lupin: SDL_BlitSurface: %s\n", SDL_GetError());
    exit(1);
  }
  SDL_UpdateRect(screen, 0, 0, 0, 0);

  SDL_Delay((random() % 4) * 20 + 50);
  SDL_FillRect(screen, &rgn, SDL_MapRGB(screen->format, black.r, black.g, black.b));
}


/* show 1 line
    pos = 0; center
    pos < 0; up
    pos > 0; down
*/
void showline(SDL_Surface* screen, iconv_t cd, TTF_Font* font, char* line, int pos, align al)
{
  char tmp2[100];
  char* p2;
  SDL_Rect dstrect;
  SDL_Surface* txt;
  int slen, dlen;
  int nconv;
  if(!strcmp(line, "")) return;
  slen = strlen(line) + 1; dlen = 100;
  p2 = tmp2;
  nconv = iconv(cd, (const char**)&line, &slen, (char**)&p2, &dlen);
  if(nconv == -1){
    fprintf(stderr, "lupin: iconv: error.\n");
    exit(1);
  }
  txt = TTF_RenderUTF8_Solid(font,tmp2,white);
  if ( txt == NULL ) {
    fprintf(stderr, "lupin: TTF_RenderUTF8_Solid: %s\n", SDL_GetError());
    exit(1);
  }
  switch(al){
  case ALIGN_LEFT:
    dstrect.x = 0;
    break;
  case ALIGN_RIGHT:
    dstrect.x = screen->w - txt->w - 40;
    break;
  case ALIGN_CENTER:
    dstrect.x = (screen->w - txt->w)/2;
    break;
  }
  if(dstrect.x < 0) dstrect.x = 0;
  dstrect.y = (screen->h - txt->h)/2 + txt->h * pos / 10;
  dstrect.w = txt->w;
  dstrect.h = txt->h;
  if ( SDL_BlitSurface(txt, NULL, screen, &dstrect) < 0 ) {
    fprintf(stderr, "lupin: SDL_BlitSurface: %s\n", SDL_GetError());
    exit(1);
  }
}

int main(int argc, char *argv[])
{
  iconv_t cd;
  SDL_Surface *screen;
  TTF_Font* font;
  int ptsize;
  int story = 0;
  char* line1, * line2;
  align alg;

#ifdef ENABLE_AUDIO
  Mix_Chunk *wav1;
  Mix_Chunk *wav2;
  int audio_rate;
  Uint16 audio_format;
  int audio_channels;
#endif
  
  srand(time(NULL));

  story = -1;
  line1 = NULL;
  if(argc > 1){
    if(isdigit(*argv[1])){
      story = atoi(argv[1]); 
    } else {
      line1 = strdup(argv[1]);
      line2 = "";
      alg = ALIGN_CENTER;
    }
  }
  if(story <= 0 || story >= 156) story = rand() % 155 + 1;

  if(line1 == NULL){
    line1 = titles[(story - 1) * 2];
    line2 = titles[(story - 1) * 2 + 1];
    fprintf(stderr, "%d\n", story);
    alg = ALIGN_LEFT;
  }
      
  /* iconv initialize */
  cd = iconv_open ("utf8", "euc-jp");
  if (cd == (iconv_t) -1){
    fprintf(stderr, "lupin: iconv_open: open error.\n");
    return 1;
  }

#ifdef ENABLE_AUDIO
  /* Initialize variables */
  audio_rate = 44100;
  audio_format = AUDIO_S16;
  audio_channels = 2;

  /* Audio Initialize */
  if ( SDL_Init(SDL_INIT_AUDIO) < 0 ) {
    fprintf(stderr, "lupin: SDL_Init(AUDIO): %s\n",SDL_GetError());
    return 1;
  }

  if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, 16) < 0) {
    fprintf(stderr, "lupin: Mix_OpenAudio: %s\n", SDL_GetError());
    return 1;
  } else {
    Mix_QuerySpec(&audio_rate, &audio_format, &audio_channels);
  }
  audio_open = 1;
  
  /* Load the requested wave file */
  wav1 = Mix_LoadWAV("lupin1.wav");
  wav2 = Mix_LoadWAV("lupin2.wav");
  
  if ( wav1 == NULL || wav2 == NULL) {
    fprintf(stderr, "lupin: Mix_LoadWAV: %s\n", SDL_GetError());
    return 1;
  }
#endif

  atexit(CleanUp);
  signal(SIGINT, exit);
  signal(SIGTERM, exit);

  /* Initialize SDL */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "lupin: SDL_Init(VIDEO): %s\n",SDL_GetError());
    return 1;
  }
  
  /* Initialize the TTF library */
  if ( TTF_Init() < 0 ) {
    fprintf(stderr, "lupin: TTF_Init: %s\n",SDL_GetError());
    return 1;
  }
  
  /* Set a 640x480x8 video mode */
  screen = SDL_SetVideoMode(640, 480, 8, SDL_SWSURFACE);
  if ( screen == NULL ) {
    fprintf(stderr, "lupin: SDL_SetVideoMode: %s\n", SDL_GetError());
    return 1;
  }
  
#if 0  /* toggle full screen */
  {
    SDL_Surface* stmp =  SDL_GetVideoSurface();
    if(!SDL_WM_ToggleFullScreen(stmp)){
      fprintf(stderr, "lupin: SDL_WM_ToggleFullScreen: %s\n", SDL_GetError());
      return 1;
    }
  }
#endif

  SDL_FillRect(screen, NULL,
	       SDL_MapRGB(screen->format, black.r, black.g, black.b));
  SDL_UpdateRect(screen, 0, 0, 0, 0);
  
  ptsize = 350;
  font = TTF_OpenFont("msmincho.ttc", ptsize);
  if ( font == NULL ) {
    fprintf(stderr, "lupin: TTF_OpenFont: %s\n", SDL_GetError());
    return 1;
  }
  TTF_SetFontStyle(font, TTF_STYLE_NORMAL);
  
  
  {
    int k, l;
    k = prepare_line(cd, font, line1);
    for(l = 0; l < k; l++){
#ifdef ENABLE_AUDIO
      Mix_PlayChannel(0, wav1, 0);
#endif
      showletter(screen, l);
    }
    SDL_Delay(50);
    k = prepare_line(cd, font, line2);
    for(l = 0; l < k; l++){
#ifdef ENABLE_AUDIO
      Mix_PlayChannel(0, wav1, 0);
#endif
      showletter(screen, l);
    }
    
  }
  
#ifdef ENABLE_AUDIO
  Mix_PlayChannel(0, wav2, 0);
#endif
  TTF_CloseFont(font);
  ptsize = 65;
  font = TTF_OpenFont("msmincho.ttc", ptsize);
  if ( font == NULL ) {
    fprintf(stderr, "lupin: TTF_OpenFont: %s\n", SDL_GetError());
    return 1;
  }
  TTF_SetFontStyle(font, TTF_STYLE_NORMAL);
  SDL_FillRect(screen, NULL,
	       SDL_MapRGB(screen->format, black.r, black.g, black.b));
  
  showline(screen, cd, font, line1, -5, alg);
  showline(screen, cd, font, line2, 5, alg);
  
  SDL_UpdateRect(screen, 0, 0, 0, 0);
#ifdef ENABLE_AUDIO
  while ( Mix_Playing(0)) SDL_Delay(100);
  Mix_CloseAudio();
#else
  SDL_Delay(5000);
#endif

  TTF_CloseFont(font);
  
  iconv_close(cd);
  
  return 0;
}
